import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1

Item{
    id: root
    height: _content.height + divider.height
    transformOrigin: Item.Bottom

    property var ops
    signal useAnswer(string answerToUse, string formulaData)
    signal labelTextUpdated(int idx, string newText)
    signal mainLabelUpdated(int idx, string newText)
    signal removeItem()

    Rectangle{
        id: _content
        width: parent.width
        height: inputs.height+units.gu(2)
        Column {
            id: inputs
            anchors.top: parent.top
            anchors.topMargin: units.gu(2)
            spacing: units.gu(0.5)
            width: parent.width - units.gu(4)
            anchors.centerIn: parent

            TextField{
                width: parent.width
                ItemStyle.delegate: Item{} // removes ubuntu shape
                text: mainLabel
                hasClearButton: false
                onTextChanged: { root.mainLabelUpdated(index, text) }
            }

            Text {
                color: "gray"
            }

            Repeater{
                id: repeater
                model: ops
                CalcLabel {
                    id: formulaLabel
                    labelText: _text
                    numbers: _number
                    operation: _operation
                    numbersColor: (answer === '' && index === repeater.model.count-1) ? "#dd4814" : "#757373"
                    numbersHeight: (answer === '' && index === repeater.model.count-1) ? units.gu(7) : units.gu(4)

                    onLabelTextChanged: {
                        root.labelTextUpdated(index, labelText)
                    }
                }
            }

            Rectangle{
                visible: answerLabel.visible
                height: units.gu(0.1)
                width: answerLabel.operationsWidth
                anchors.right: parent.right
                color: '#757373'
            }

            CalcLabel {
                id: answerLabel
                visible: (answer !== '')
                numbers: answer
                labelReadOnly: true
                labelText: i18n.tr("Total")
                numbersColor: "#dd4814"
                numbersHeight: units.gu(7)

                MouseArea {
                    id: screenMA
                    enabled: !isLastItem
                    anchors.fill: parent
                    onClicked: {
                        if (answer.indexOf('error') === -1)
                            root.useAnswer(answer, formula_data)
                    }
                }
            }
        }

        Behavior on x {
            NumberAnimation {
                duration: 200
            }
        }


        MouseArea {
            anchors.fill: parent
            drag.target: _content
            drag.axis: Drag.XAxis
            drag.minimumX: isLastItem ? 0 : -_content.width
            drag.maximumX: isLastItem ? 0 : _content.width
            propagateComposedEvents: true

            function handleDrag() {
                if (Math.abs(_content.x) < parent.width/3) {
                    _content.x = 0
                }
                else {
                    if (_content.x < 0)
                        _content.x = -_content.width;
                    else
                        _content.x = _content.width;
                    root.removeItem()
                }
            }

            onReleased: handleDrag()
            onCanceled: handleDrag()
        }
    }

    Divider {
        id: divider
        visible: !isLastItem
        width: parent.width
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
    }
}
