import QtQuick 2.0
import Ubuntu.Components 0.1
import "../engine.js" as CALC

Page {

    property var context: new CALC.Context;
    property var scanner;
    property string answer: ''
    property var screenFormula: [{_text:'', _operation:'', _number:''}]
    property var engineFormula: ['']

    function formulaPush(visual, engine) {
        try{
            scanner = new CALC.Scanner(engineFormula.join('') + engine, context);
        } catch(exception) {
            console.log(exception)
            return;
        }
        if(scanner.tokens.length > 1){
            switch(scanner.tokens.last().type){
            case CALC.T_PLUS:
            case CALC.T_MINUS:
            case CALC.T_DIV:
            case CALC.T_TIMES:
                if(scanner.tokens[scanner.tokens.length-2].type & CALC.T_OPERATOR) return;
                screenFormula.push({_text:'', _operation: '', _number:''});
                screenFormula[screenFormula.length-1]._operation = visual;
                engineFormula[engineFormula.length-1] += engine;
                engineFormula.push('');
                formulaView.currentOperatorsChanged();
                return;
            case CALC.T_UNARY_PLUS:
            case CALC.T_UNARY_MINUS:
                if(scanner.tokens[scanner.tokens.length-2].type === CALC.T_UNARY_MINUS ||
                        scanner.tokens[scanner.tokens.length-2].type === CALC.T_UNARY_PLUS) return;
                break;
            }
        }
        screenFormula[screenFormula.length-1]._number += visual;
        engineFormula[engineFormula.length-1] += engine;
        formulaView.currentOperatorsChanged();
    }

    function changeSign(){
        if(scanner !== undefined && scanner.tokens.length > 0){
            switch(scanner.tokens.last().type){
            case CALC.T_NUMBER:
                if(scanner.tokens.length > 1 && scanner.tokens[scanner.tokens.length-2].type === CALC.T_UNARY_MINUS){
                    scanner.tokens.splice(scanner.tokens.length-2,1);
                    scanner.tokens.last().value = scanner.tokens.last().value * -1
                }
                scanner.tokens.last().value = scanner.tokens.last().value * -1
                screenFormula[screenFormula.length-1]._number = (scanner.tokens.last().value).toString();
                engineFormula[engineFormula.length-1] = (scanner.tokens.last().value).toString();
                break;
            case CALC.T_UNARY_MINUS:
                scanner.tokens.pop();
                screenFormula[screenFormula.length-1]._number = '';
                engineFormula.pop();
                break;
            default:
                formulaPush('-', '-');
            }
        } else { formulaPush('-','-'); }
        formulaView.currentOperatorsChanged();
    }

    function calculate() {
        var result = 0;
        try{
            result = new CALC.Parser(scanner).reduce(context);
            if (result === Number.POSITIVE_INFINITY)
                result = '∞';
            else if (result === Number.NEGATIVE_INFINITY)
                result = '-∞';
        } catch(exception) {
            if(exception instanceof CALC.DivisionByZeroError){
                result = "division by zero error";
            } else if(exception instanceof SyntaxError){
                result = "";
            }
        }
        answer = result;
    }

    function remove(index){
        screenFormula.splice(index, 1);
        engineFormula.splice(index, 1);
        formulaView.currentOperatorsChanged();
    }

    function clear(){
        answer = '';
        screenFormula = [{_text:'', _operation:'', _number:''}];
        engineFormula = [''];
        scanner = new CALC.Scanner(engineFormula.join(''), context);
        formulaView.currentOperatorsChanged();
    }

    function addFromMemory(answerToAdd, formulaData) {
        if (answerToAdd !== '' && answerToAdd.indexOf('error') === -1 && answerToAdd.indexOf('∞') === -1) {
            for (var i = 0; i < answerToAdd.length; i++)
                formulaPush(answerToAdd[i], answerToAdd[i], answerToAdd[i] === '.' ? 'real' : 'number')
        }
    }

    Item {
        anchors.fill: parent

        ListView {
            id: formulaView
            anchors.fill: parent
            verticalLayoutDirection: ListView.BottomToTop
            clip: true

            signal currentOperatorsChanged()
            property string currentAnswer: answer

            function addCurrentToMemory() {
                if (answer !== '') {
                    var currentDate = new Date();
                    if (memory.get(0).mainLabel.length == 0)
                        memory.get(0).mainLabel = currentDate.toString();
                    memory.get(0).isLastItem = false
                    memory.setProperty(0, "timeStamp", currentDate.getTime());
                    memory.insert(0,{'dbId': -1, 'answer': '', 'operators': [{_text:'', _operation:'', _number:''}], 'isLastItem': true, 'mainLabel': ''});
                    positionViewAtBeginning();
                }
            }

            function showError() {
                animateError.start()
            }

            PropertyAnimation {id: animateError; target: formulaView; properties: "color"; from: "#FFA0A0"; to: "#FFFFFF"; duration: 100}

            remove: Transition{
                NumberAnimation {property: "opacity"; to: 0; duration: 200 }
            }

            displaced: Transition{
                SequentialAnimation{
                    NumberAnimation {property: "scale"; to: 0.9; duration: 50 }
                    NumberAnimation {property: "y"; duration: 300 }
                    NumberAnimation {property: "scale"; to: 1; duration: 50 }
                }
            }

            onCurrentOperatorsChanged: {
                memory.get(0).operators = screenFormula
                positionViewAtBeginning();
            }

            onCurrentAnswerChanged: {
                memory.get(0).answer = currentAnswer
            }

            model: Memory{
                id: memory
                Component.onCompleted: {
                    storage.getCalculations(function(calc){
                        memory.append(calc);
                    });
                }
            }

            header: CalcKeyboard {}

            delegate: Screen {
                id: screen
                width: formulaView.width
                ops: operators

                onUseAnswer: addFromMemory(answerToUse, formulaData)
                onMainLabelUpdated: {
                    memory.get(index).mainLabel = newText;
                }
                onLabelTextUpdated: {
                    if(index === 0){
                        screenFormula[idx]._text = newText;
                        return;
                    }
                    memory.get(index).operators.get(idx)._text = newText;
                }
                onRemoveItem: {
                    storage.removeCalculation(memory.get(index));
                    memory.remove(index)
                }
            }

            Scrollbar {
                flickableItem: formulaView
                align: Qt.AlignTrailing
            }

            Component.onCompleted: {
                currentOperatorsChanged();
            }

            Component.onDestruction: {
                //update the storage
                var calculations = []
                for(var i=1; i<memory.count; i++){
                    var operators = memory.get(i).operators;
                    var newop = [];
                    if(operators.count !== undefined){
                    for(var j=0; j< operators.count; j++){
                        newop.push(operators.get(j));
                    }
                    }else{
                        newop = operators
                    }
                    var newElement = {'dbId': memory.get(i).dbId,
                        'isLastItem': false,
                        'mainLabel': memory.get(i).mainLabel,
                        'answer': memory.get(i).answer,
                        'operators': newop};
                    calculations.push({"calc": newElement, "date": memory.get(i).timeStamp})
                }
                storage.saveCalculations(calculations)
            }
        }
    }
}
